<?php

/*******************************************************************************
 *  
 *  KeetagCaller version 2.2
 *  
 *  
 * === What is this file ? ===
 * 
 *  This PHP library gives you a ready to use solution to easily integrate 
 *  the Keetag text converter (https://keetag.net) directly in your own 
 *  website or application.
 *  
 *
 *  This library makes you able to call easily and efficiently the Keetag web API, 
 *  without passing by the graphical interface of the Keetag website.
 * 
 * 
 * === How can I use this file ? ===
 * 
 *  This library is provided with a sample file 'demo.php' which shows 
 *  how to use it. If you do not have the demo file, you can download it
 *  on the Keetag website : https://keetag.net
 * 
 *  The code is explicited by commentaries. If you need more detailed informations, 
 *  see the online documentation at https://keetag.net
 * 
 * 
 * === Can I freely use this file ? ===
 * 
 *  Yes! You can freely integrate this library into your own site,
 *  and/or modify it, or inspire you for your own scripts.
 * 
 ******************************************************************************/


class KeetagCaller
{
    
    function __construct()
    {
        $this->set_urls('https://keetag.net');
        $this->output_format = 'array';
        // Will contain the options to send to the API
        $this->options = [];
    }
    
    
    /**
     * The main method of the class: converts the text by calling the API,
     * by sending the options defined with set_options().
     * 
     * @param string $text_to_convert   (Optional) The text you want to convert. 
     *                                  Not required if you have already defined your text
     *                                  with the method set_option('text_to_convert')
     * 
     * @return string   The converted text and informations about the conversion.
     *                  The format will be JSON or array or object, depending on 
     *                  what you set with $this->output_format. 
     */
    public function convert_text($text_to_convert=NULL, $orig_language_id=NULL, $destin_language_id=NULL)
    {
        
        // Note the non-strict comparison to treat equally NULL, 0, empty string and FALSE
        if ($text_to_convert != NULL) {
            
            // Allows to send the text to convert directly in the method convert_text() :
            //     $keetag->convert_text($my_text);
            // rather than using the method set_option() (one more line) :
            //     $this->set_option('text_to_convert', $my_text);
            //     $keetag->convert_text();
            $this->set_option('text_to_convert', $text_to_convert);
        }
        
        if ($orig_language_id != NULL) {
            
            // Allows to set the original language of the text directly with convert_text()
            $this->set_option('orig_language_id', $orig_language_id);
        }
        
        if ($destin_language_id != NULL) {
            
            // Allows to set the original language of the text directly with convert_text()
            $this->set_option('destin_language_id', $destin_language_id);
        }
        
        return $this->get_api_output($this->api_url_converter, $this->options);
    }
    
    
    /**
     * Gets the list of the languages supported by Keetag.
     */
    public function get_languages()
    {
        
        return $this->get_api_output($this->api_url_languages);
    }
    
    
    /**
     * Generic method to set options that will be sent to the online API.
     */
    public function set_option($option_name, $option_value)
    {
        
        return $this->options[$option_name] = $option_value;
    }
    
    
    /**
     * Permet de changer la racine des URL d'appel à l'API si besoin.
     * Facultatif. Sera utile si on fait tourner le site sur un autre hébergement 
     * que keetag.net.
     * 
     * @param string $url   La racine du site voulue.
     *                      Ex : 'http://mysite.com'
     */
    public function set_root_url($root)
    {
        
        $this->set_urls($root);
    }
    
    
    /**
     * Sets the output format of the datas (JSON, array, object...).
     * See the change_json_format() method above to know the available formats.
     */
    public function set_output_format($format)
    {
        
        return $this->output_format = $format;
    }
    
    
    /**
     * Sets the URLs of all the Keetag APIs.
     */
    private function set_urls($root)
    {
        
        $this->api_url_converter    = $root.'/en/api/v1/converter';
        $this->api_url_languages    = $root.'/en/api/v1/languages';
    }
    
    
    /**
     * Gets and transforms the raw résult of the API :
     * > do the safety checks (checks that the API url exists, etc.)
     * > formats the API result in the specified format (raw JSON, array...)
     * 
     * @param string $api_url   L'url de l'API de Keetag à appeler.
     *                          Ex : $this->api_url_converter
     *                          Ex : $this->api_url_languages
     * 
     * @param array $datas_to_send  Les données à envoyer en POST à l'API de Keetag
     * 
     * @return json|array
     */
    private function get_api_output($api_url, $datas_to_send=[])
    {
        
        // Sends the text to the online API and gets the JSON result
        $raw_server_output = $this->get_raw_api_output($api_url, $datas_to_send);
        
        // If the API web page is not available (wrong url, timeout...)
        if ($raw_server_output === FALSE) {
            
            $raw_server_output = '[KeetagCaller] Check the API URL you set : ' .$api_url;
            
            $formatted_server_output = $this->emergency_output_on_failure(
                                                $raw_server_output,
                                                'unreachable_api_webpage'
                                                );
        }
        else {
            
            // Sets the output format of the datas (JSON, array, object...).
            // See the change_json_format() method below to know the available formats.
            $formatted_server_output = $this->change_json_format($raw_server_output,
                                                                 $this->output_format
                                                                 );

            // If the server returned an invalid JSON, replaces it by an alternative output
            if(json_last_error() !== 0) {

                $formatted_server_output = $this->emergency_output_on_failure(
                                                    $raw_server_output,
                                                    'converter_crash'
                                                    );
            }
        }
        
        return $formatted_server_output;
    }
    
    
    /**
     * Sends the text to convert and the defined options to the online API, 
     * then gets the JSON result returned by the server.
     * This method call and returns the raw API without any check, contrary to
     * the method get_api_output().
     * 
     * @param string $api_url   L'url de l'API de Keetag à appeler.
     *                          Ex : $this->api_url_converter
     *                          Ex : $this->api_url_languages
     * 
     * @param array $datas_to_send  Les données à envoyer en POST à l'API de Keetag
     */
    private function get_raw_api_output($api_url, $datas_to_send)
    {
        
        // The datas will be sent to the API web page through the POST method
        $stream = [
            'http' => [
                'header'  => "Content-type: application/x-www-form-urlencoded\r\n"
                            // VERY IMPORTANT : sets a user agent. If no user agent
                            // is sent, the Keetag server will return a "403/forbidden" error
                            // so you wouldn't be able to access the API.
                            ."User-Agent: Mozilla/4.0 (compatible; MSIE 6.0)\r\n",
                'method'  => 'POST',
                'content' => http_build_query($datas_to_send)
                ]
            ];
        
        // Reads the result returned by the Keetag API.
        // Note that file_get_contents() works on most of servers but is quite limited.
        // If you need a more powerful tool, use the cURL library if it's available
        // on your server. See http://php.net/manual/en/curl.examples-basic.php
        return file_get_contents(   $api_url, 
                                    FALSE,
                                    stream_context_create($stream)
                                    );
    }
    
    
    /**
     * Converts the JSON-formatted datas returned by the API
     * to an exploitable PHP format: an array or an object.
     */
    private function change_json_format($json, $format)
    {
        
        switch ($format) {
            
            case 'array':
                $result = json_decode($json, TRUE);
                break;
            
            case 'object': 
                $result = json_decode($json);
                break;
            
            // All other values will return the JSON as is.
            default:
                $result = $json;
                // We execute a json_decode() here too, although this won't be 
                // the return value in this case. This is essential to 
                // check if the JSON is valid or not (see the method
                // emergency_output_on_failure()).
                json_decode($json);
                break;
        }
        
        return $result;
    }
    
    
    /**
     * A minimalist but valid JSON structure to replace the JSON 
     * normally returned by the Keetag API. Useful if the server returned 
     * an invalid JSON for an unexpected reason (this should never happen but 
     * must be considered). This will: 
     *      1) avoid crashing the user scripts that read the API
     *      2) allow the debugging by showing server errors (or other 
     *         causes of failure) that would not be visible without 
     *         this method.
     * 
     * @return  The minimalist JSON replacement.
     */
    private function emergency_output_on_failure($raw_server_output, $error_alias)
    {
        
        $error_message = [
            'converter_crash'           => 'The Keetag server returned a malformed JSON string. '
                                            . 'The output you are reading has been generated by '
                                            . 'your local KeetagCaller library to replace the broken JSON. '
                                            . 'You will find the output of the server '
                                            . 'in the field "output_for_debugging".',
            'unreachable_api_webpage'   =>  'The Keetag API web page could not be reached. '
                                            . 'The output you are reading has been generated by '
                                            . 'your local KeetagCaller library to replace the missing one.'
            ];
        
        $emergency_output = json_encode([
            'error' => [
                'error_alias'           => $error_alias,
                'error_message'         => $error_message[$error_alias],
                'output_for_debugging'  => $raw_server_output
                ]
            ]);

        // Converts the emergency output to the same format than the one
        // initially planned for the normal output (JSON, array or object)
        $result = $this->change_json_format($emergency_output,
                                            $this->output_format
                                            );
        
        return $result;
    }
}
